/*
The software in this package is distributed under the GNU General
Public License version 2 (with a special exception described below).

A copy of GNU General Public License (GPL) is included in this distribution,
in the file COPYING.GPL.

As a special exception, if other files instantiate templates or use macros
or inline functions from this file, or you compile this file and link it
with other works to produce a work based on this file, this file
does not by itself cause the resulting work to be covered
by the GNU General Public License.

However the source code for this file must still be made available
in accordance with section (3) of the GNU General Public License.

This exception does not invalidate any other reasons why a work based
on this file might be covered by the GNU General Public License.
*/
#include "link/statusnotifiercommand.h"

#include <cstdlib>

#include <iostream>

#include <boost/foreach.hpp>

#include <logfunc.hpp>
#include <stringfunc.hxx>

#include "boost_assert_handler.h"

using namespace std;
using I2n::Logger::GlobalLogger;

//-----------------------------------------------------------------------------
// StatusNotifierCommand
//-----------------------------------------------------------------------------

const string StatusNotifierCommand::StatusToken = "${status}";

//-----------------------------------------------------------------------------

StatusNotifierCommand::StatusNotifierCommand(
        const string &status_notifier_cmd
) :
    CommandStr( status_notifier_cmd ),
    CommandStrOriginal( status_notifier_cmd )
{
}

StatusNotifierCommand::~StatusNotifierCommand()
{
}

/**
 * @brief Replaces the token by the value in the notifier command.
 *
 * @param token actual string in the command.
 * @param value string to replace the token.
 *
 * @return true if the token was find and replaced, or false if it was not
 * found, neither replaced.
 */
bool StatusNotifierCommand::set_token_value(
        const string &token,
        const string &value
)
{
    BOOST_ASSERT( !token.empty() );
    BOOST_ASSERT( !value.empty() );

    // use a fresh copy of the original string where token is not replaced yet
    CommandStr = CommandStrOriginal;

    const size_t token_begin_pos = CommandStr.find( token );
    if ( token_begin_pos == string::npos )
    {
        return false; // token string not found!
    }

    const size_t token_size = token.length();
    (void) CommandStr.replace( token_begin_pos, token_size, value );

    // assert the token is no longer within the command string
    BOOST_ASSERT( CommandStr.find( token ) == string::npos );

    return true;
}

/**
 * @brief Tries to execute the notify command.
 *
 * @return true if the command could be executed. false otherwise.
 */
bool StatusNotifierCommand::execute() const
{
    // Check if users wants notification at all
    if (CommandStr.empty())
        return true;

    string app = get_application_string();
    string args = get_arguments_string();
    string secure_command = app + " " + args;

    GlobalLogger.info() << "- Command: " << secure_command
            << endl;

    int ret = system( secure_command.c_str() );
    if ( ret != 0 )
        return false;

    return true;
}

string StatusNotifierCommand::get_application_string() const
{
    // retrieve the application name from within the command string
    const size_t app_begin_pos = 0;
    size_t app_end_pos = CommandStr.find( " " );
    if ( app_end_pos == string::npos )
    {
        app_end_pos = CommandStr.find( "\n" );
        if ( app_end_pos == string::npos )
        {
            return "invalid-app";
        }
    }

    return CommandStr.substr( app_begin_pos, app_end_pos );
}

string StatusNotifierCommand::get_arguments_string() const
{
    // retrieve the arguments string from within the command string
    const size_t args_begin_pos = CommandStr.find( " " ) + 1;
    const size_t args_end_pos = CommandStr.length();
    std::list< string > args = I2n::split_string(
            CommandStr.substr( args_begin_pos, args_end_pos ),
            " ", true);
    BOOST_FOREACH( string arg, args )
        arg = escape_shellarg( arg );
    return I2n::join_string( args, " " );
}
